"""
Application about dialog.
-------------------------

"""

import sys
from xml.sax.saxutils import escape

from AnyQt.QtWidgets import (
    QDialog, QDialogButtonBox, QVBoxLayout, QLabel, QApplication
)
from AnyQt.QtCore import Qt

from .. import config

ABOUT_TEMPLATE = """\
<center>
<h4>{name}</h4>
<p>Version: {version}</p>
</center>
"""


class AboutDialog(QDialog):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)

        if sys.platform == "darwin":
            self.setAttribute(Qt.WA_MacSmallSize, True)

        self.__setupUi()

    def __setupUi(self):
        layout = QVBoxLayout()
        label = QLabel(self)

        pixmap, _ = config.splash_screen()

        label.setPixmap(pixmap)

        layout.addWidget(label, Qt.AlignCenter)

        name = QApplication.applicationName()
        version = QApplication.applicationVersion()

        text = ABOUT_TEMPLATE.format(
            name=escape(name),
            version=escape(version),
        )
        # TODO: Also list all known add-on versions??.
        text_label = QLabel(text)
        layout.addWidget(text_label, Qt.AlignCenter)

        buttons = QDialogButtonBox(
            QDialogButtonBox.Close, Qt.Horizontal, self)
        layout.addWidget(buttons)
        buttons.rejected.connect(self.accept)
        layout.setSizeConstraint(QVBoxLayout.SetFixedSize)
        self.setLayout(layout)



"""
Scheme Info editor widget.

"""
import typing
from typing import Optional

from AnyQt.QtWidgets import (
    QWidget, QDialog, QLabel, QTextEdit, QCheckBox, QFormLayout,
    QVBoxLayout, QHBoxLayout, QDialogButtonBox, QSizePolicy
)
from AnyQt.QtCore import Qt

from ..gui.lineedit import LineEdit
from ..gui.utils import StyledWidget_paintEvent, StyledWidget

if typing.TYPE_CHECKING:
    from ..scheme import Scheme


class SchemeInfoEdit(QWidget):
    """Scheme info editor widget.
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.scheme = None  # type: Optional[Scheme]
        self.__schemeIsUntitled = True
        self.__setupUi()

    def __setupUi(self):
        layout = QFormLayout()
        layout.setRowWrapPolicy(QFormLayout.WrapAllRows)
        layout.setFieldGrowthPolicy(QFormLayout.ExpandingFieldsGrow)

        self.name_edit = LineEdit(self)
        self.name_edit.setPlaceholderText(self.tr("untitled"))
        self.name_edit.setSizePolicy(QSizePolicy.Expanding,
                                     QSizePolicy.Fixed)
        self.desc_edit = QTextEdit(self)
        self.desc_edit.setTabChangesFocus(True)

        layout.addRow(self.tr("Title"), self.name_edit)
        layout.addRow(self.tr("Description"), self.desc_edit)

        self.setLayout(layout)

    def setScheme(self, scheme):
        # type: (Scheme) -> None
        """Set the scheme to display/edit

        """
        self.scheme = scheme
        if not scheme.title:
            self.name_edit.setText(self.tr("untitled"))
            self.name_edit.selectAll()
            self.__schemeIsUntitled = True
        else:
            self.name_edit.setText(scheme.title)
            self.__schemeIsUntitled = False
        self.desc_edit.setPlainText(scheme.description or "")

    def commit(self):
        # type: () -> None
        """
        Commit the current contents of the editor widgets back to the scheme.
        """
        if self.scheme is None:
            return

        if self.__schemeIsUntitled and \
            self.name_edit.text() == self.tr("untitled"):
            # 'untitled' text was not changed
            name = ""
        else:
            name = self.name_edit.text().strip()

        description = self.desc_edit.toPlainText().strip()
        self.scheme.title = name
        self.scheme.description = description

    def paintEvent(self, event):
        return StyledWidget_paintEvent(self, event)

    def title(self):
        # type: () -> str
        return self.name_edit.text().strip()

    def description(self):
        # type: () -> str
        return self.desc_edit.toPlainText().strip()


class SchemeInfoDialog(QDialog):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.scheme = None  # type: Optional[Scheme]
        self.__autoCommit = True

        self.__setupUi()

    def __setupUi(self):
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        self.editor = SchemeInfoEdit(self)
        self.editor.layout().setContentsMargins(20, 20, 20, 20)
        self.editor.layout().setSpacing(15)
        self.editor.setSizePolicy(QSizePolicy.MinimumExpanding,
                                  QSizePolicy.MinimumExpanding)

        heading = self.tr("Workflow Info")
        heading = "<h3>{0}</h3>".format(heading)
        self.heading = QLabel(heading, self, objectName="heading")

        # Insert heading
        self.editor.layout().insertRow(0, self.heading)

        self.buttonbox = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
            Qt.Horizontal,
            self
            )

        # Insert button box
        self.editor.layout().addRow(self.buttonbox)

        widget = StyledWidget(self, objectName="auto-show-container")
        check_layout = QHBoxLayout()
        check_layout.setContentsMargins(20, 10, 20, 10)
        self.__showAtNewSchemeCheck = \
            QCheckBox(self.tr("Show when I make a New Workflow."),
                      self,
                      objectName="auto-show-check",
                      checked=False,
                      )

        check_layout.addWidget(self.__showAtNewSchemeCheck)
        check_layout.addWidget(
               QLabel(self.tr("You can also edit Workflow Info later "
                              "(File -> Workflow Info)."),
                      self,
                      objectName="auto-show-info"),
               alignment=Qt.AlignRight)
        widget.setLayout(check_layout)
        widget.setSizePolicy(QSizePolicy.MinimumExpanding,
                             QSizePolicy.Fixed)

        if self.__autoCommit:
            self.buttonbox.accepted.connect(self.editor.commit)

        self.buttonbox.accepted.connect(self.accept)
        self.buttonbox.rejected.connect(self.reject)

        layout.addWidget(self.editor, stretch=10)
        layout.addWidget(widget)

        self.setLayout(layout)

    def setShowAtNewScheme(self, checked):
        # type: (bool) -> None
        """
        Set the 'Show at new scheme' check state.
        """
        self.__showAtNewSchemeCheck.setChecked(checked)

    def showAtNewScheme(self):
        # type: () -> bool
        """
        Return the check state of the 'Show at new scheme' check box.
        """
        return self.__showAtNewSchemeCheck.isChecked()

    def setAutoCommit(self, auto):
        # type: (bool) -> None
        if self.__autoCommit != auto:
            self.__autoCommit = auto
            if auto:
                self.buttonbox.accepted.connect(self.editor.commit)
            else:
                self.buttonbox.accepted.disconnect(self.editor.commit)

    def setScheme(self, scheme):
        # type: (Scheme) -> None
        """Set the scheme to display/edit.
        """
        self.scheme = scheme
        self.editor.setScheme(scheme)



"""
"""
import sys

from types import ModuleType


def make_module(name, package="", **namespace):
    mod = ModuleType(name)
    mod.__package__ = package or ""
    if package:
        mod.__name__ = "{}.{}".format(package, name)
    else:
        mod.__name = name
    mod.__dict__.update(namespace)
    return mod

Test = make_module(
    "Test", __package__,
    NAME="Test",
    DESCRIPTION="This is a test.",
    LONG_DESCRIPTION="This. Is. A. Test.",
    PRIORITY=3
)
Test.__path__ = []


constants = make_module(
    "constants", __package__, NAME="Constants"
)
constants.__path__ = []

constants.zero = make_module(
    "zero", constants.__name__,
    NAME="Zero",
    OUTPUTS=[("value", int)],
    CATEGORY="Constants",
    zero=None,
)

constants.one = make_module(
    "one", constants.__name__,
    NAME="One",
    OUTPUTS=[("value", int)],
    CATEGORY="Constants",
    one=None,
)

operators = make_module(
    "operators", __package__, NAME="Operators"
)
operators.__path__ = []

operators.add = make_module(
    "add", operators.__name__,
    NAME="Add",
    INPUTS=[("left", int, "set_left"),
            ("right", int, "set_right")],
    OUTPUTS=[("value", int)],
    CATEGORY="Operators",
    add=None
)

operators.sub = make_module(
    "sub", operators.__name__,
    NAME="Subtract",
    INPUTS=[("left", int, "set_left"),
            ("right", int, "set_right")],
    OUTPUTS=[("value", int)],
    CATEGORY="Operators",
    sub=None,
)

operators.mult = make_module(
    "mult", operators.__name__,
    NAME="Multiply",
    INPUTS=[("left", int, "set_left"),
            ("right", int, "set_right")],
    OUTPUTS=[("value", int)],
    CATEGORY="Operators",
    mult=None,
)

operators.div = make_module(
    "div", operators.__name__,
    NAME="Divide",
    INPUTS=[("left", int, "set_left"),
            ("right", int, "set_right")],
    OUTPUTS=[("value", int)],
    CATEGORY="Operators",
    div=None
)


def set_up_package(package):
    sys.modules[package.__name__] = package
    for val in package.__dict__.values():
        if isinstance(val, ModuleType):
            sys.modules[val.__name__] = val


def tear_down_package(package):
    for val in package.__dict__.values():
        if isinstance(val, ModuleType):
            if val.__name__ in sys.modules:
                del sys.modules[val.__name__]
    if package.__name__ in sys.modules:
        del sys.modules[package.__name__]


def set_up_modules():
    set_up_package(constants)
    set_up_package(operators)


def tear_down_modules():
    tear_down_package(constants)
    tear_down_package(operators)


def small_testing_registry():
    """Return a small registry with a few widgets for testing.
    """
    from ..description import (
        WidgetDescription, CategoryDescription, InputSignal, OutputSignal
    )
    from .. import WidgetRegistry

    registry = WidgetRegistry()

    const_cat = CategoryDescription(
        "Constants", background="light-orange")

    zero = WidgetDescription(
        "zero", "zero", "Constants",
        qualified_name="zero",
        package=__package__,
        outputs=[OutputSignal("value", "int", id="val")])

    one = WidgetDescription(
        "one", "one", "Constants",
        qualified_name="one",
        package=__package__,
        outputs=[OutputSignal("value", "int")])

    unit = WidgetDescription(
        "unit", "unit", "Constants",
        qualified_name="unit",
        package=__package__,
        outputs=[OutputSignal("value", "tuple")])

    op_cat = CategoryDescription(
        "Operators", background="grass")

    add = WidgetDescription(
        "add", "add", "Operators",
        qualified_name="add",
        package=__package__,
        inputs=[InputSignal("left", "int", "set_left"),
                InputSignal("right", "int", "set_right", id="droite")],
        outputs=[OutputSignal("result", "int")]
    )
    sub = WidgetDescription(
        "sub", "sub", "Operators",
        qualified_name="sub",
        package=__package__,
        inputs=[InputSignal("left", "int", "set_left"),
                InputSignal("right", "int", "set_right")],
        outputs=[OutputSignal("result", "int")]
    )
    mult = WidgetDescription(
        "mult", "mult", "Operators",
        qualified_name="mult",
        package=__package__,
        inputs=[InputSignal("left", "int", "set_left"),
                InputSignal("right", "int", "set_right")],
        outputs=[OutputSignal("result", "int")]
    )
    div = WidgetDescription(
        "div", "div", "Operators",
        qualified_name="div",
        package=__package__,
        inputs=[InputSignal("left", "int", "set_left"),
                InputSignal("right", "int", "set_right")],
        outputs=[OutputSignal("result", "int")]
    )
    negate = WidgetDescription(
        "negate", "negate", "Operators",
        qualified_name="negate",
        package=__package__,
        inputs=[InputSignal("value", "int", "set_value")],
        outputs=[OutputSignal("result", "int")],
    )
    struct_cat = CategoryDescription(
        "Structure", background="red")

    cons = WidgetDescription(
        "cons", "cons", "Structure",
        qualified_name="cons",
        package=__package__,
        inputs=[InputSignal("first", "object", "set_first"),
                InputSignal("second", "object", "set_second")],
        outputs=[OutputSignal("cons", "tuple")]
    )
    decons = WidgetDescription(
        "decons", "decons", "Structure",
        qualified_name="decons",
        package=__package__,
        inputs=[InputSignal("cons", "tuple", "set_cons")],
        outputs=[OutputSignal("first", "object", doc="First matched"),
                 OutputSignal("second", "object", doc="Second matched"),
                 OutputSignal("empty", "tuple", doc="No match")]
    )

    registry.register_category(const_cat)
    registry.register_category(op_cat)
    registry.register_category(struct_cat)

    registry.register_widget(zero)
    registry.register_widget(one)
    registry.register_widget(unit)
    registry.register_widget(add)
    registry.register_widget(sub)
    registry.register_widget(mult)
    registry.register_widget(div)
    registry.register_widget(negate)

    registry.register_widget(cons)
    registry.register_widget(decons)

    return registry
